@@ -1,8 +1,8 @@
 from typing import Tuple
 
 from pythautomata.automata.wheighted_automaton_definition.probabilistic_deterministic_finite_automaton import \
-    ProbabilisticDeterministicFiniteAutomaton
-
+    ProbabilisticDeterministicFiniteAutomaton as PDFA, ProbabilisticDeterministicFiniteAutomaton
+from pythautomata.model_comparators.wfa_tolerance_comparison_strategy import WFAToleranceComparator
 from pymodelextractor.teachers.probabilistic_teacher import ProbabilisticTeacher
 from pymodelextractor.learners.observation_tree_learners.pdfa_quantization_n_ary_tree_learner import \
      PDFAQuantizationNAryTreeLearner
@@ -10,18 +10,20 @@ from pymodelextractor.learners.learning_result import LearningResult
 from pymodelextractor.exceptions.query_length_exceeded_exception import QueryLengthExceededException
 from pymodelextractor.exceptions.number_of_states_exceeded_exception import NumberOfStatesExceededException
 from pymodelextractor.utils.time_bound_utilities import timeout
-
+from pymodelextractor.learners.observation_table_learners.observation_table import epsilon
+from pythautomata.automata.wheighted_automaton_definition.weighted_state import WeightedState
 
 class BoundedPDFAQuantizationNAryTreeLearner(PDFAQuantizationNAryTreeLearner):
-    def __init__(self, comparator, max_states, max_query_length, max_seconds_run=None):
-        super().__init__(comparator)
+    def __init__(self, partitioner, max_states, max_query_length, max_seconds_run=None, generate_partial_hipothesis = False):
+        super().__init__(partitioner)
         self._max_states = max_states
         self._max_query_length = max_query_length
         self._max_seconds_run = max_seconds_run
         self._exceeded_max_states = False
         self._exceeded_max_mq_length = False
         self._exceded_time_bound = False
-        self._history = []
+        self._history = []        
+        self._generate_partial_hipothesis = generate_partial_hipothesis
 
     def _perform_equivalence_query(self, model):
         self._history.append(model)
@@ -49,6 +51,11 @@ class BoundedPDFAQuantizationNAryTreeLearner(PDFAQuantizationNAryTreeLearner):
         except QueryLengthExceededException:
             print("QueryLengthExceeded")
             self._exceeded_max_mq_length = True
+
+        if not self._exceeded_max_states and self._generate_partial_hipothesis:
+            partial_hipothesis = self.partial_hipothesis()
+            self._history.append(partial_hipothesis)
+
         hist = list(self._history)
         result = self._learning_results_for(hist[-1] if len(hist) > 0 else None)
         result.info['NumberOfStatesExceeded'] = self._exceeded_max_states
@@ -61,3 +68,25 @@ class BoundedPDFAQuantizationNAryTreeLearner(PDFAQuantizationNAryTreeLearner):
         if not ret[0]:
             self._tree.max_query_length = self._max_query_length
         return ret
+
+    def partial_hipothesis(self) -> PDFA:
+        states = {}
+        symbols = list(self._alphabet.symbols)
+        symbols.sort()
+        updated_tree = True
+        
+        for leaf_str, leaf in self._tree.leaves.items():
+            initial_weight = 1 if leaf_str == epsilon else 0
+            terminal_symbol_probability = leaf.probabilities[self.terminal_symbol]
+            state = WeightedState(leaf_str, initial_weight, terminal_symbol_probability)
+            states[leaf_str] = state
+        unknown_state =  WeightedState(self._tree.unknown_leaf, 0, 0)     
+        states[self._tree.unknown_leaf] = unknown_state
+        for access_string, state in states.items():
+            for symbol in symbols:
+                access_string_of_transition, _ = self._tree.sift(access_string + symbol)
+                state.add_transition(symbol, states[access_string_of_transition],
+                                        self._tree.leaves[access_string].probabilities[symbol])
+        comparator = WFAToleranceComparator()
+        states = set(states.values())
+        return PDFA(self._alphabet, states, self.terminal_symbol, comparator=comparator, check_is_probabilistic = False)
\ No newline at end of file
